import './index.css';
import nameGenerator from './name-generator';
import isDef from './is-def';



// Store/retrieve the name in/from a cookie.
const cookies = document.cookie.split(';');
console.log(cookies)
let wsname = cookies.find(function(c) {
  if (c.match(/wsname/) !== null) return true;
  return false;
});
if (isDef(wsname)) {
  wsname = wsname.split('=')[1];
} else {
  wsname = nameGenerator();
  document.cookie = "wsname=" + encodeURIComponent(wsname);
}

// Set the name in the header
document.querySelector('header>p').textContent = "CANVAS WEBSOCKET";

// Create a WebSocket connection to the server
const ws = new WebSocket("ws://" + window.location.host+ "/socket");
const myName = decodeURI(wsname);
const myColor = getRandomColor();
var drawing = false;
ws.binaryType = "arraybuffer";

// We get notified once connected to the server
ws.onopen = (event) => {
  console.log("We are connected.");
};

window.onload = function() {
	var myCanvas = document.getElementById("myCanvas");
	var ctx = myCanvas.getContext("2d");

  var coordinates = "";



  // Fill Window Width and Height
  myCanvas.width = 1080;
	myCanvas.height = 720;

	// Set Background Color
    ctx.fillStyle="#fff";
    ctx.fillRect(0,0,myCanvas.width,myCanvas.height);

    // Mouse Event Handlers
	if(myCanvas){
		var isDown = false;
		var canvasX, canvasY;
    var strokeColor = myColor;
    ctx.lineWidth = 5;
    var myInputColor = document.getElementById("color");
    myInputColor.value = myColor;
    var myInputName = document.getElementById("name");
    myInputName.value = myName;

		myCanvas.onmousedown = function(e){
      drawing = true;
			isDown = true;
			ctx.beginPath();
			canvasX = e.pageX - myCanvas.offsetLeft;
			canvasY = e.pageY - myCanvas.offsetTop;
      coordinates+="&"+canvasX+","+canvasY;
			ctx.moveTo(canvasX, canvasY);
		};

		myCanvas.onmousemove = function(e){
			if(isDown !== false) {
				canvasX = e.pageX - myCanvas.offsetLeft;
				canvasY = e.pageY - myCanvas.offsetTop;
        coordinates+="&"+canvasX+","+canvasY;
				ctx.lineTo(canvasX, canvasY);
				ctx.strokeStyle = strokeColor;
				ctx.stroke();
			}
		};

		myCanvas.onmouseup = function(e){
      coordinates+="&!";
      drawing = false;
			isDown = false;
			ctx.closePath();

      var data = {
        "name": myName,
        "color": myColor,
        "canvas": coordinates
      };

      ws.send(JSON.stringify(data));

      console.log("up");
		};
	}

	// Touch Events Handlers
	var draw = {
		started: false,
		start: function(evt) {

			ctx.beginPath();
			ctx.moveTo(
				evt.touches[0].pageX,
				evt.touches[0].pageY
			);

			this.started = true;

		},
		move: function(evt) {

      drawOther();
			if (this.started) {
				ctx.lineTo(
					evt.touches[0].pageX,
					evt.touches[0].pageY
				);

				ctx.strokeStyle = strokeColor;
				ctx.lineWidth = 5;
				ctx.stroke();
			}

		},
		end: function(evt) {
			this.started = false;
		}
	};

	// Touch Events
	myCanvas.addEventListener('touchstart', draw.start, false);
	myCanvas.addEventListener('touchend', draw.end, false);
	myCanvas.addEventListener('touchmove', draw.move, false);

	// Disable Page Move
	document.body.addEventListener('touchmove',function(evt){
		evt.preventDefault();
	},false);

  document.getElementById('clear').addEventListener('click', function() {
    ctx.clearRect(0, 0, myCanvas.width, myCanvas.height);
    coordinates="";
  }, false);

  document.getElementById('otherCanvas').addEventListener('click', function() {
    drawOther();
  }, false);

  window.setInterval(function(){
    drawOther();
  }, 500);


  function drawOther(){

    var myCanvas2 = document.getElementById("myCanvas");
    var ctx2 = myCanvas2.getContext("2d");

    var e = document.getElementById("otherCanvas");
    if(e.options[e.selectedIndex].text !== "" && e.options[e.selectedIndex].text!== decodeURI(wsname) && drawing===false){

      var strUser = e.options[e.selectedIndex].text;
      var colorUser = arrayColor[strUser];
      var coordUser = arrayData[strUser].split("&");
      console.log("coordUser.length :"+ coordUser.length);

      var oldX = parseInt(coordUser[0].substring(0,coordUser[0].indexOf(",")));
      var oldY = parseInt(coordUser[0].substring(coordUser[0].indexOf(",")+1))

      ctx2.beginPath();
      for(var i in coordUser){
        if(coordUser[i]=="!"){
          console.log(parseInt(i)+1);
          if(parseInt(i)+2 < coordUser.length){
            console.log("+1");
            ctx2.closePath();
            oldX = parseInt(coordUser[parseInt(i)+1].substring(0,coordUser[parseInt(i)+1].indexOf(",")));
            oldY = parseInt(coordUser[parseInt(i)+1].substring(coordUser[parseInt(i)+1].indexOf(",")+1))
            ctx2.beginPath();
          }
        }else{
          var newX = parseInt(coordUser[i].substring(0,coordUser[i].indexOf(",")));
          var newY = parseInt(coordUser[i].substring(coordUser[i].indexOf(",")+1))

          ctx2.strokeStyle = colorUser;
          ctx2.lineWidth = 5;
          ctx2.moveTo(oldX,oldY);
          ctx2.lineTo(newX,newY);

          oldX = newX;
          oldY = newY;

          ctx2.stroke();
        }
      }
			ctx2.closePath();
    }
  }


};

var arrayNames = new Array();
var arrayColor = [];
var arrayData = [];

ws.onmessage = (event) => {
  //console.log(event.data);
  var name = event.data.substring(event.data.lastIndexOf("[")+1,event.data.lastIndexOf("]"));
  arrayNames.push(name);
  arrayColor[name]=event.data.substring(event.data.lastIndexOf("#"),event.data.lastIndexOf("#")+7);
  arrayData[name]=event.data.substring(event.data.indexOf("&"),event.data.lastIndexOf('"'));
  addSelect();

};



function addSelect(){
  if(arrayNames.length > 0 ){

    var arrayOptions = new Array();

    var select = document.getElementById("otherCanvas");
    var i;
    for (i = 0; i < select.options.length; i++) {
        arrayOptions.push(select.options[i].value);
    }

    arrayNames.forEach(function(name) {
      if(!arrayOptions.includes(name)){
        var option = document.createElement("option");
        option.text = name;
        select.add(option);
      }
    });
  }
}

function getRandomColor() {
  var letters = '0123456789ABCDEF';
  var color = '#';
  for (var i = 0; i < 6; i++) {
    color+=letters[Math.floor(Math.random()*16)];
  }
  return color;
}
